@@ -2,7 +2,6 @@
 
 require 'socket'
 require 'timeout'
-require 'thread'
 require 'io/wait'
 
 begin
@@ -189,7 +188,7 @@ class Resolv
         unless @initialized
           @name2addr = {}
           @addr2name = {}
-          open(@filename, 'rb') {|f|
+          File.open(@filename, 'rb') {|f|
             f.each {|line|
               line.sub!(/#.*/, '')
               addr, hostname, *aliases = line.split(/\s+/)
@@ -735,47 +734,35 @@ class Resolv
         def initialize(*nameserver_port)
           super()
           @nameserver_port = nameserver_port
-          @initialized = false
-          @mutex = Thread::Mutex.new
-        end
-
-        def lazy_initialize
-          @mutex.synchronize {
-            next if @initialized
-            @initialized = true
-            @socks_hash = {}
-            @socks = []
-            @nameserver_port.each {|host, port|
-              if host.index(':')
-                bind_host = "::"
-                af = Socket::AF_INET6
-              else
-                bind_host = "0.0.0.0"
-                af = Socket::AF_INET
-              end
-              next if @socks_hash[bind_host]
-              begin
-                sock = UDPSocket.new(af)
-              rescue Errno::EAFNOSUPPORT
-                next # The kernel doesn't support the address family.
-              end
-              @socks << sock
-              @socks_hash[bind_host] = sock
-              sock.do_not_reverse_lookup = true
-              DNS.bind_random_port(sock, bind_host)
-            }
+          @socks_hash = {}
+          @socks = []
+          nameserver_port.each {|host, port|
+            if host.index(':')
+              bind_host = "::"
+              af = Socket::AF_INET6
+            else
+              bind_host = "0.0.0.0"
+              af = Socket::AF_INET
+            end
+            next if @socks_hash[bind_host]
+            begin
+              sock = UDPSocket.new(af)
+            rescue Errno::EAFNOSUPPORT
+              next # The kernel doesn't support the address family.
+            end
+            sock.do_not_reverse_lookup = true
+            DNS.bind_random_port(sock, bind_host)
+            @socks << sock
+            @socks_hash[bind_host] = sock
           }
-          self
         end
 
         def recv_reply(readable_socks)
-          lazy_initialize
           reply, from = readable_socks[0].recvfrom(UDPSize)
           return reply, [from[3],from[1]]
         end
 
         def sender(msg, data, host, port=Port)
-          lazy_initialize
           sock = @socks_hash[host.index(':') ? "::" : "0.0.0.0"]
           return nil if !sock
           service = [host, port]
@@ -787,14 +774,9 @@ class Resolv
         end
 
         def close
-          @mutex.synchronize {
-            if @initialized
-              super
-              @senders.each_key {|service, id|
-                DNS.free_request_id(service[0], service[1], id)
-              }
-              @initialized = false
-            end
+          super
+          @senders.each_key {|service, id|
+            DNS.free_request_id(service[0], service[1], id)
           }
         end
 
@@ -818,32 +800,20 @@ class Resolv
           super()
           @host = host
           @port = port
-          @mutex = Thread::Mutex.new
-          @initialized = false
-        end
-
-        def lazy_initialize
-          @mutex.synchronize {
-            next if @initialized
-            @initialized = true
-            is_ipv6 = @host.index(':')
-            sock = UDPSocket.new(is_ipv6 ? Socket::AF_INET6 : Socket::AF_INET)
-            @socks = [sock]
-            sock.do_not_reverse_lookup = true
-            DNS.bind_random_port(sock, is_ipv6 ? "::" : "0.0.0.0")
-            sock.connect(@host, @port)
-          }
-          self
+          is_ipv6 = host.index(':')
+          sock = UDPSocket.new(is_ipv6 ? Socket::AF_INET6 : Socket::AF_INET)
+          @socks = [sock]
+          sock.do_not_reverse_lookup = true
+          DNS.bind_random_port(sock, is_ipv6 ? "::" : "0.0.0.0")
+          sock.connect(host, port)
         end
 
         def recv_reply(readable_socks)
-          lazy_initialize
           reply = readable_socks[0].recv(UDPSize)
           return reply, nil
         end
 
         def sender(msg, data, host=@host, port=@port)
-          lazy_initialize
           unless host == @host && port == @port
             raise RequestError.new("host/port don't match: #{host}:#{port}")
           end
@@ -854,15 +824,10 @@ class Resolv
         end
 
         def close
-          @mutex.synchronize do
-            if @initialized
-              super
-              @senders.each_key {|from, id|
-                DNS.free_request_id(@host, @port, id)
-              }
-              @initialized = false
-            end
-          end
+          super
+          @senders.each_key {|from, id|
+            DNS.free_request_id(@host, @port, id)
+          }
         end
 
         class Sender < Requester::Sender # :nodoc:
@@ -876,7 +841,6 @@ class Resolv
 
       class MDNSOneShot < UnconnectedUDP # :nodoc:
         def sender(msg, data, host, port=Port)
-          lazy_initialize
           id = DNS.allocate_request_id(host, port)
           request = msg.encode
           request[0,2] = [id].pack('n')
@@ -886,7 +850,6 @@ class Resolv
         end
 
         def sender_for(addr, msg)
-          lazy_initialize
           @senders[msg.id]
         end
       end
@@ -965,7 +928,7 @@ class Resolv
         nameserver = []
         search = nil
         ndots = 1
-        open(filename, 'rb') {|f|
+        File.open(filename, 'rb') {|f|
           f.each {|line|
             line.sub!(/[#;].*/, '')
             keyword, *args = line.split(/\s+/)
@@ -1557,12 +1520,12 @@ class Resolv
         def initialize(data)
           @data = data
           @index = 0
-          @limit = data.length
+          @limit = data.bytesize
           yield self
         end
 
         def inspect
-          "\#<#{self.class}: #{@data[0, @index].inspect} #{@data[@index..-1].inspect}>"
+          "\#<#{self.class}: #{@data.byteslice(0, @index).inspect} #{@data.byteslice(@index..-1).inspect}>"
         end
 
         def get_length16
@@ -1581,7 +1544,7 @@ class Resolv
 
         def get_bytes(len = @limit - @index)
           raise DecodeError.new("limit exceeded") if @limit < @index + len
-          d = @data[@index, len]
+          d = @data.byteslice(@index, len)
           @index += len
           return d
         end
@@ -1609,9 +1572,9 @@ class Resolv
 
         def get_string
           raise DecodeError.new("limit exceeded") if @limit <= @index
-          len = @data[@index].ord
+          len = @data.getbyte(@index)
           raise DecodeError.new("limit exceeded") if @limit < @index + 1 + len
-          d = @data[@index + 1, len]
+          d = @data.byteslice(@index + 1, len)
           @index += 1 + len
           return d
         end
@@ -1634,7 +1597,7 @@ class Resolv
           d = []
           while true
             raise DecodeError.new("limit exceeded") if @limit <= @index
-            case @data[@index].ord
+            case @data.getbyte(@index)
             when 0
               @index += 1
               if save_index
@@ -1671,7 +1634,13 @@ class Resolv
           name = self.get_name
           type, klass, ttl = self.get_unpack('nnN')
           typeclass = Resource.get_class(type, klass)
-          res = self.get_length16 { typeclass.decode_rdata self }
+          res = self.get_length16 do
+            begin
+              typeclass.decode_rdata self
+            rescue => e
+              raise DecodeError, e.message, e.backtrace
+            end
+          end
           res.instance_variable_set :@ttl, ttl
           return name, ttl, res
         end
@@ -2634,7 +2603,7 @@ class Resolv
     def each_address(name)
       name = Resolv::DNS::Name.create(name)
 
-      return unless name.to_a.last == 'local'
+      return unless name.to_a.last.to_s == 'local'
 
       super(name)
     end
